#!/bin/bash

# This script replays all ethtool commands from the
# /etc/network/interfaces file. It will be called from switchd
# init script to work around a problem of ethtool settings
# getting reset when switcd is restarted
# 
SCRIPTNAME=$0
VERBOSE=$1
[ -z $VERBOSE ] && VERBOSE="no"

old_IFS=$IFS
IFS=$'\n'
lines=( $(ifquery -a | grep -e "^[[:space:]]*iface" -e "^[[:space:]]*link-" | sed 's/^\s*//') )
IFS=$old_IFS

# if its boottime, skip the ethool replay, because
# the networking init script will set the ethtool settings
runlevel=$(/sbin/runlevel 2>/dev/null | cut -d " " -f 2)
[ $runlevel == 'S' -o $runlevel == 'unknown' ] && exit 0

iface=
i=0
len=${#lines[@]}
while [ $i -lt $len ];
do
    key=$(echo ${lines[$i]} | tr -s " " | cut -d " " -f 1)
    val=$(echo ${lines[$i]} | tr -s " " | cut -d " " -f 2)
    if [ "$key" == "iface" ]; then
        iface=$val

        # lookahead for ethtool parameters
        j=$(($i+1))
        eargs=''
        while [ $j -lt $len -a "${lines[$j]:0:5}" != "iface" ];
        do
            ekey=$(echo ${lines[$j]} | tr -s " " | cut -d " " -f 1)
            eval=$(echo ${lines[$j]} | tr -s " " | cut -d " " -f 2)
            if [ "$ekey" == "link-speed" ]; then
                eargs="$eargs speed $eval"
            elif [ "$ekey" == "link-autoneg" ]; then
                eargs="$eargs speed $eval"
            elif [ "$ekey" == "link-duplex" ]; then
                eargs="$eargs duplex $eval"
            fi
            j=$((j+1))
        done
        if [ "$eargs" != '' ]; then
            cmd="/sbin/ethtool -s $iface $eargs"
            [ $VERBOSE != "no" ] && echo "$cmd"
            cmdout=$($cmd 2>&1)
            if [ $? -ne 0 ]; then
                echo "warn: $cmdout"
                # use logger if needed
                #/usr/bin/logger -s -i -t $SCRIPTNAME
            fi
        fi
        i=$((j-1))
    fi
    i=$((i+1))
done
